#
# DeepLabCut Toolbox (deeplabcut.org)
# © A. & M.W. Mathis Labs
# https://github.com/DeepLabCut/DeepLabCut
#
# Please see AUTHORS for contributors.
# https://github.com/DeepLabCut/DeepLabCut/blob/master/AUTHORS
#
# Licensed under GNU Lesser General Public License v3.0
#
import json
import os

# just expand this list when adding new models:
MODELOPTIONS = [
    "full_human",
    "full_cat",
    "full_dog",
    "primate_face",
    "mouse_pupil_vclose",
    "horse_sideview",
    "full_macaque",
    "superanimal_topviewmouse",
    "superanimal_quadruped",
]


def _get_dlclibrary_path():
    """Get path of where dlclibrary (this repo) is currently running"""
    import importlib.util

    return os.path.split(importlib.util.find_spec("dlclibrary").origin)[0]


def _load_model_names():
    """Load URLs and commit hashes for available models."""
    from ruamel.yaml import YAML

    fn = os.path.join(_get_dlclibrary_path(), "dlcmodelzoo", "modelzoo_urls.yaml")
    with open(fn) as file:
        return YAML().load(file)


def parse_available_supermodels():
    libpath = _get_dlclibrary_path()
    json_path = os.path.join(libpath, "dlcmodelzoo", "superanimal_models.json")
    with open(json_path) as file:
        return json.load(file)


def download_huggingface_model(modelname, target_dir=".", remove_hf_folder=True):
    """
    Download a DeepLabCut Model Zoo Project from Hugging Face

    Parameters
    ----------
    modelname : string
        Name of the ModelZoo model. For visualizations see: http://www.mackenziemathislab.org/dlc-modelzoo
    target_dir : directory (as string)
        Directory where to store the model weights and pose_cfg.yaml file
    remove_hf_folder : bool, default True
        Whether to remove the directory structure provided by HuggingFace after downloading and decompressing data into DeepLabCut format.
    """
    from huggingface_hub import hf_hub_download
    import tarfile
    from pathlib import Path

    neturls = _load_model_names()
    if modelname not in neturls:
        raise ValueError(f"`modelname` should be one of: {', '.join(modelname)}.")

    print("Loading....", modelname)
    url = neturls[modelname].split("/")
    repo_id, targzfn = url[0] + "/" + url[1], str(url[-1])

    hf_hub_download(repo_id, targzfn, cache_dir=str(target_dir))

    # Create a new subfolder as indicated below, unzipping from there and deleting this folder
    hf_folder = f"models--{url[0]}--{url[1]}"
    hf_path = os.path.join(
        hf_folder,
        "snapshots",
        str(neturls[modelname + "_commit"]),
        targzfn,
    )

    filename = os.path.join(target_dir, hf_path)
    with tarfile.open(filename, mode="r:gz") as tar:
        for member in tar:
            if not member.isdir():
                fname = Path(member.name).name
                tar.makefile(member, os.path.join(target_dir, fname))

    if remove_hf_folder:
        import shutil

        shutil.rmtree(os.path.join(target_dir, hf_folder))



#
# DeepLabCut Toolbox (deeplabcut.org)
# © A. & M.W. Mathis Labs
# https://github.com/DeepLabCut/DeepLabCut
#
# Please see AUTHORS for contributors.
# https://github.com/DeepLabCut/DeepLabCut/blob/master/AUTHORS
#
# Licensed under GNU Lesser General Public License v3.0
#

__version__ = "0.0.3"
VERSION = __version__



#
# DeepLabCut Toolbox (deeplabcut.org)
# © A. & M.W. Mathis Labs
# https://github.com/DeepLabCut/DeepLabCut
#
# Please see AUTHORS for contributors.
# https://github.com/DeepLabCut/DeepLabCut/blob/master/AUTHORS
#
# Licensed under GNU Lesser General Public License v3.0
#
import dlclibrary
import os
import pytest
from dlclibrary.dlcmodelzoo.modelzoo_download import MODELOPTIONS


def test_download_huggingface_model(tmp_path_factory, model="full_cat"):
    folder = tmp_path_factory.mktemp("temp")
    dlclibrary.download_huggingface_model(model, str(folder))

    assert os.path.exists(folder / "pose_cfg.yaml")
    assert any(f.startswith("snapshot-") for f in os.listdir(folder))
    # Verify that the Hugging Face folder was removed
    assert not any(f.startswith("models--") for f in os.listdir(folder))


def test_download_huggingface_wrong_model():
    with pytest.raises(ValueError):
        dlclibrary.download_huggingface_model("wrong_model_name")


def test_parse_superanimal_models():
    dict_ = dlclibrary.parse_available_supermodels()
    assert "superanimal_quadruped" in dict_
    assert "superanimal_topviewmouse" in dict_


@pytest.mark.skip
@pytest.mark.parametrize("model", MODELOPTIONS)
def test_download_all_models(tmp_path_factory, model):
    test_download_huggingface_model(tmp_path_factory, model)



#
# DeepLabCut Toolbox (deeplabcut.org)
# © A. & M.W. Mathis Labs
# https://github.com/DeepLabCut/DeepLabCut
#
# Please see AUTHORS for contributors.
# https://github.com/DeepLabCut/DeepLabCut/blob/master/AUTHORS
#
# Licensed under GNU Lesser General Public License v3.0
#
import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="dlclibrary",
    version="0.0.3",
    author="A. & M. Mathis Labs",
    author_email="alexander@deeplabcut.org",
    description="Lightweight library supporting universal functions for the DeepLabCut ecosystem",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/DeepLabCut/DLClib",
    install_requires=[
        "huggingface_hub",
        "ruamel.yaml>=0.15.0",
    ],
    packages=setuptools.find_packages(),
    data_files=[
        (
            "dlclibrary",
            [
                "dlclibrary/dlcmodelzoo/modelzoo_urls.yaml",
                "dlclibrary/dlcmodelzoo/superanimal_models.json",
                "dlclibrary/dlcmodelzoo/superanimal_configs/superquadruped.yaml",
                "dlclibrary/dlcmodelzoo/superanimal_configs/supertopview.yaml",
            ],
        )
    ],
    include_package_data=True,
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU Lesser General Public License v3 (LGPLv3)",
        "Operating System :: OS Independent",
    ],
)
